/****************************************************
 * 有機ＥＬキャラクタディスプレイモジュール　１６×２行
 * SO1602AWWB-UC-WB-U　（電源電圧3.3V）
 *　 表示テストプログラム
 * 　使用マイコン　PIC16F1827
 *   使用コンパイラ　XC8　Ver.1.45
 * 　I2C　割り当て
 * 　SDA  :  SDA1:PORTB(1) pin7
 * 　SCL  :  SCL1:PORTB(4) pin10
 * 
 *                                2021／1／25
 ****************************************************/

#include    <xc.h>
#include    <stdio.h>
#include    "i2c_lib4.h"
#define 	_XTAL_FREQ 8000000 //PICのクロックをHzで設定(8MHz)

/************** コンフィグレーション設定   *****************/
// コンフィギュレーション１の設定
#pragma config FOSC = INTOSC    // 内部ｸﾛｯｸを使用する(INTOSC)
#pragma config WDTE = OFF       // ｳｵｯﾁﾄﾞｯｸﾞﾀｲﾏｰ無し(OFF)
#pragma config PWRTE = ON       // 電源ONから64ms後にﾌﾟﾛｸﾞﾗﾑを開始する(ON)
#pragma config MCLRE = ON       // 外部ﾘｾｯﾄ信号を使用
#pragma config CP = OFF         // ﾌﾟﾛｸﾞﾗﾑﾒﾓﾘｰを保護しない(OFF)
#pragma config CPD = OFF        // ﾃﾞｰﾀﾒﾓﾘｰを保護しない(OFF)
#pragma config BOREN = ON       // 電源電圧降下常時監視機能ON(ON)
#pragma config CLKOUTEN = OFF   // CLKOUTﾋﾟﾝをRA6ﾋﾟﾝで使用する(OFF)
#pragma config IESO = OFF       // 外部・内部ｸﾛｯｸの切替えでの起動はなし(OFF)
#pragma config FCMEN = OFF      // 外部ｸﾛｯｸ監視しない(FCMEN_OFF)

// コンフィギュレーション２の設定
#pragma config WRT = OFF        // Flashﾒﾓﾘｰを保護しない(OFF)
#pragma config PLLEN = OFF      // 動作クロックを32MHzでは動作させない(OFF)
#pragma config STVREN = ON      // スタックがオーバフローやアンダーフローしたらリセットをする(ON)
#pragma config BORV = HI        // 電源電圧降下常時監視電圧(2.5V)設定(HI)
#pragma config LVP = OFF        // 低電圧プログラミング機能使用しない(OFF)


/*************** サブ関数   **************************/
void LCD_dat(char);                     // １文字表示
void LCD_cmd(char);                     // コマンド出力
void LCD_clr(void);                     // 全消去
void LCD_int(void);                     // 初期化

void contrast_max(void);                //コントラスト調整

/**************** メイン開始  ************************/
void main(void){

    OSCCON = 0b01110000;           // PLL:OFF, 内部クロック8MHzで駆動
    ANSELB = 0b00000000;           // RB7(AN6)-RB1(AN11)は全て0:デジタルI/Oとする
    TRISB  = 0b11111111;           // PortBを全て1:入力I/Oにする
    OPTION_REGbits.nWPUEN = 0;     // 0:PORTB内部プルアップ利用に設定
    WPUBbits.WPUB1 = 1;            // RB1をI2C通信のウィークプルアップ
    WPUBbits.WPUB4 = 1;            // RB4をI2C通信のウィークプルアップ
    SSPCON1 = 0b00101000;          // I2Cマスターモード指定
    SSPSTAT = 0b00000000;          // I2C STATUSの設定
    SSPADD  = 19;                  // I2Cクロック周波数100KHz(=(19+1)*4/8MHz)
    LCD_int();                     // LCD初期化
    contrast_max();                //　コントラスト調整 
    
    // ------------------------------------------------------

    LCD_cmd(0x0c);                  //　ブリンク非表示
    __delay_ms(2);                  //コマンド送信後は2ms以上のウエイトが必要
   
    LCD_cmd(0x40);                  //CGRAM　アドレスセット　　
    __delay_ms(2);                  //コマンド送信後は2ms以上のウエイトが必要

    LCD_dat(0b00101010);
    LCD_dat(0b00101010);
    LCD_dat(0b00101010);
    LCD_dat(0b00101010);    
    LCD_dat(0b00101010);
    LCD_dat(0b00101010);
    LCD_dat(0b00101010);
    LCD_dat(0b00101010);
    
    LCD_cmd(0x48);                  //CGRAM　アドレスセット　
    __delay_ms(2);                 
    LCD_dat(0b00011101);
    LCD_dat(0b00010101);
    LCD_dat(0b00010101);
    LCD_dat(0b00010101);
    LCD_dat(0b00010101);
    LCD_dat(0b00010101);
    LCD_dat(0b00010001);
    LCD_dat(0b00011111);
    
    LCD_cmd(0x50);                  //CGRAM　アドレスセット　
    __delay_ms(2);                  
    LCD_dat(0b00011111);
    LCD_dat(0b00000000);
    LCD_dat(0b00011111);
    LCD_dat(0b00000000);
    LCD_dat(0b00011111);
    LCD_dat(0b00000000);
    LCD_dat(0b00011111);
    LCD_dat(0b00000000);
    
    LCD_cmd(0x58);                  //CGRAM　アドレスセット　
    __delay_ms(2);                  //コマンド送信後は2ms程度のウエイトが必要
    LCD_dat(0b00010101);
    LCD_dat(0b00001010);
    LCD_dat(0b00010101);
    LCD_dat(0b00001010);
    LCD_dat(0b00010101);
    LCD_dat(0b00001010);
    LCD_dat(0b00010101);
    LCD_dat(0b00001010);
    
   LCD_cmd(0x60);                  //CGRAM　アドレスセット　
    __delay_ms(2);                 
    LCD_dat(0b00010101);
    LCD_dat(0b00010101);
    LCD_dat(0b00010101);
    LCD_dat(0b00010101);
    LCD_dat(0b00010101);
    LCD_dat(0b00010101);
    LCD_dat(0b00010101);
    LCD_dat(0b00010101);
  
    while(1){    
        LCD_clr(); 

        LCD_cmd(0x80);                 //1行目に移動 
        __delay_ms(2);
        printf("     Hello !   ");
        LCD_cmd(0xA0);                 //2行目に移動
        __delay_ms(2);
        printf("  2021/01/25   ");

        __delay_ms(3000);

        LCD_clr(); 
        LCD_cmd(0x80);                  //1行目に移動
        __delay_ms(2);
        LCD_dat(0x00);                  //CGRAMキャラ               
        LCD_dat(0x01);                  //CGRAMキャラ 
        LCD_dat(0x02);                  //CGRAMキャラ
        LCD_dat(0x03);                  //CGRAMキャラ
        LCD_dat(0x04);                  //CGRAMキャラ
    
        LCD_dat(0x20);                  
        LCD_dat(0x11);                  
        LCD_dat(0x12);                  
        LCD_dat(0x13);                  
        LCD_dat(0x14);                  
        LCD_dat(0x15);                  
    
        LCD_dat(0xF2);
        LCD_dat(0x43);
        LCD_dat(0xE5);
        LCD_dat(0xE6);
        LCD_dat(0x24);
    
        LCD_cmd(0xA0);                  //2行目に移動
        __delay_ms(2);

        LCD_dat(0xF4);
        LCD_dat(0xF5);        
        LCD_dat(0xF6);
        LCD_dat(0xF7);
        LCD_dat(0xF8);
        LCD_dat(0xF9);        
        LCD_dat(0xFA);
        LCD_dat(0xFB);
        LCD_dat(0xFC);
        LCD_dat(0xFD);        
        LCD_dat(0xFF);
        LCD_dat(0xFE);
        LCD_dat(0x32);
        LCD_dat(0x33);
        LCD_dat(0x34);    
        LCD_dat(0x1E);
    
        __delay_ms(3000);
    
        LCD_cmd(0x80);      //1行目に移動     
        __delay_ms(2);
        
        LCD_dat('A');
        LCD_dat('B');
        LCD_dat('C');
        LCD_dat('D');
        LCD_dat('E');
        LCD_dat('F');
        LCD_dat('G');
        LCD_dat('H');
        LCD_dat('I');
        LCD_dat('J');
        LCD_dat('K');
        LCD_dat('L');
        LCD_dat('M');
        LCD_dat('N');
        LCD_dat('O');
        LCD_dat('P');        
        LCD_cmd(0x2C);      //2C=高文字
        __delay_ms(3000);

        LCD_cmd(0x28);      //　28=ノーマル
        
  }    
}
//-------- １文字表示 --------------------------------------
void LCD_dat(char chr){
    
    CmdI2C(0x78, 0x40, chr);

    __delay_us(60);                // 60μsec
}
//-------- コマンド出力 -----------------------------------------
void LCD_cmd(char cmd){

    CmdI2C(0x78, 0x00, cmd);
    if(cmd & 0xFC)                 // 上位６ビットに１がある命令
        __delay_us(60);            // 60usec
    else
        __delay_ms(3);             // 3msec　ClearおよびHomeコマンド
}
//-------- 全消去 ------------------------------------------------
void LCD_clr(void){
    LCD_cmd(0x01);                 //Clearコマンド出力
}

//-------- 初期化 -----------------------------------------------
void LCD_int(void){
    __delay_ms(400);               // 電源安定するまでの時間
    LCD_cmd(0x01);                 // 8bit 2行 表示命令モード
    __delay_ms(2);
    LCD_cmd(0x02);                 // Display on Cursor=0 Blink=0
    __delay_ms(2);
    LCD_cmd(0x0f);                 // Entry Inc/Dec=1 Shift=0
    __delay_ms(2);
    LCD_cmd(0x01);                 // Clear Display
    __delay_ms(2);
}
//------- コントラスセット　　------------------------------------
void contrast_max(void){
    LCD_cmd(0x2a);      //RE=1
    LCD_cmd(0x79);      //SD=1
    LCD_cmd(0x81);      //コントラストセット
    LCD_cmd(0xFF);      //輝度MAX
    LCD_cmd(0x78);      //SDを0に戻す
    LCD_cmd(0x28);      //2C=高文字　28=ノーマル
    __delay_ms(100);
}

/************************************************************
 * LCD 表示器　printf 用 
 ***********************************************************/
void putch(unsigned char byte){
        
      LCD_dat(byte);
}