/******************************************
*  I2C 通信ライブラリ
******************************************/
#include <htc.h>
#include "i2c_lib4.h"

/*************************************
* I2C1バイトデータ送信
**************************************/
void SendI2C(unsigned Adrs, unsigned char Data)
{
    IdleI2C();
    SSPCON2bits.SEN = 1;   		//
    while(SSPCON2bits.SEN);		//
    SendByte(Adrs);             //
    SendByte(Data);             //
    SSPCON2bits.PEN = 1;   		//
    while(SSPCON2bits.PEN);		//
}
/****************************************
* I2Cバスを使ってコマンド送信
*****************************************/
void CmdI2C(unsigned Adrs, unsigned Reg, unsigned char Data)
{
    IdleI2C();
    SSPCON2bits.SEN = 1;   		// スタート出力　
    while(SSPCON2bits.SEN);		// スタート終了
    SendByte(Adrs);             // アドレス＋送信モード送信
    SendByte(Reg);              // レジスタ指定送信
    SendByte(Data);             // コマンドデータ送信
    SSPCON2bits.PEN = 1;   		// ストップ出力
    while(SSPCON2bits.PEN);		// ストップ終了待ち
}
/** アイドル待ちサブ関数　***/
void IdleI2C(void){
    while(SSPCON2bits.SEN || SSPCON2bits.PEN || SSPCON2bits.RCEN
            || SSPCON2bits.ACKEN || SSPSTATbits.R_nW);
}
/****************************************
*I2Cで１バイト送信
*****************************************/
void SendByte(unsigned char ch){
    SSPBUF = ch;                //　データセット送信開始 
    while(SSPSTATbits.BF);		//  送信終了待ち
    while(SSPCON2bits.ACKSTAT); //　ACK返信待ち
    IdleI2C();                  // アイドル待ち
}
/*****************************************
* I2Cバスを使ってｎバイト受信
******************************************/
void GetDataI2C(unsigned char Adrs, unsigned char *Buffer, unsigned char Cnt)
{
    unsigned char i;

    IdleI2C();                  // アイドル待ち
    SSPCON2bits.SEN = 1;   		// スタート出力
    while(SSPCON2bits.SEN);		// スタート終了待ち
    SendByte(Adrs+1);           // アドレス＋受信モード送信    
    for(i=0; i<Cnt; i++)
        Buffer[i] = RcvByte(0); // １バイト受信　ACK応答
    Buffer[i] = RcvByte(1);     // 最終バイト受信NACK応答
    SSPCON2bits.PEN = 1;   		// ストップ出力
    while(SSPCON2bits.PEN);		// ストップ終了待ち
}
/****************************************
* I2Cで１バイト受信　ACK/NACK　返送
*****************************************/
unsigned char RcvByte(unsigned char Ans){
    unsigned char data;
    
    SSPCON2bits.ACKDT = Ans;		// ACK(0)/Nack（１）設定
    SSPCON2bits.RCEN = 1;           // 受信許可　RCENセット
    while(!SSPSTATbits.BF);         // 受信待ちBFチェック
    SSPCON2bits.ACKEN = 1;          // ASK返送
    data = SSPBUF;                  // 受信データ取得
    SSPCON1bits.SSPOV = 0;          // エラークリア
    IdleI2C();                      // アイドル待ち
    return(data);                   //　受信データを返す
}
